# Mago: The Oxidized PHP Toolchain

**Mago** is a comprehensive toolchain for PHP that helps developers write better code. Inspired by the Rust ecosystem, Mago brings convenience, reliability, and an exceptional developer experience to PHP projects of all sizes.

## Features

- Analyze and Lint: Identify issues in your codebase with customizable rules and plugins.
- Automated Fixes: Apply fixes for lint issues to maintain a clean codebase.
- Consistent Formatting: Standardize your code style with powerful formatting tools.
- Semantic Checks: Ensure code correctness with robust semantic analysis.
- AST Visualization: Explore your code’s structure with abstract syntax tree (AST) parsing.

## Get Started

Ready to dive in? Visit the [Mago Documentation](https://mago.carthage.software/) for everything you need to know, including:

- [Getting Started](https://mago.carthage.software/#/getting-started/)
- [Installation Guide](https://mago.carthage.software/#/getting-started/installation)
- [Configuration Guide](https://mago.carthage.software/#/getting-started/configuration)
- [CLI Commands](https://mago.carthage.software/#/getting-started/cli)

## Contributing

Mago is a community-driven project, and we’d love for you to get involved! Whether you're reporting bugs,
suggesting features, writing documentation, or submitting code, every contribution makes a difference.

To learn more, see our [CONTRIBUTING.md](./CONTRIBUTING.md).

Join the discussion, ask questions, and connect with the Mago community on [Discord](https://discord.gg/mwyyjr27eu).

## Inspiration

Mago is inspired by several tools and projects that have significantly contributed to the development community:

- [Clippy](https://github.com/rust-lang/rust-clippy): A collection of lints to catch common mistakes and improve your Rust code.
- [OXC](https://github.com/oxc-project/oxc/): A JavaScript toolchain written in Rust.
- [php-rust-tools/parser](https://github.com/php-rust-tools/parser/): A PHP parser written in Rust, which influenced our parsing approach.
- [slackhq/hakana](https://github.com/slackhq/hakana/): A static analysis tool for HackLang written in Rust, by the creator of [Psalm](https://github.com/vimeo/psalm).

These tools have inspired us and helped shape Mago's design and functionality.

## Acknowledgements

We would like to acknowledge the following PHP tools that have greatly helped hundreds of thousands of PHP developers in their journey,
ourselves included:

- [PHP CS Fixer](https://github.com/PHP-CS-Fixer/PHP-CS-Fixer): A tool to automatically fix PHP Coding Standards issues.
- [Psalm](https://github.com/vimeo/psalm): A static analysis tool for finding errors in PHP applications.
- [PHPStan](https://github.com/phpstan/phpstan): PHP Static Analysis Tool.
- [PHP_CodeSniffer](https://github.com/squizlabs/PHP_CodeSniffer): Detects violations of a defined set of coding standards.

While Mago is intended to be a comprehensive toolchain that may eventually replace some of these tools,
we deeply appreciate their contributions and the foundation they have built for the PHP community.

## License

Mago is licensed under either of

- MIT License (MIT) - see [LICENSE-MIT](./LICENSE-MIT) file for details
- Apache License, Version 2.0 (Apache-2.0) - see [LICENSE-APACHE](./LICENSE-APACHE) file for details

at your option.

---

Thank you for your interest in Mago. We look forward to sharing our progress and collaborating with the community as the project evolves.
